@@ -32,9 +32,10 @@ namespace arrow {
 class ARROW_EXPORT NullBuilder : public ArrayBuilder {
  public:
   explicit NullBuilder(MemoryPool* pool = default_memory_pool(),
-                       int64_t alignment = kDefaultBufferAlignment)
+                       int64_t ARROW_ARG_UNUSED(alignment) = kDefaultBufferAlignment)
       : ArrayBuilder(pool) {}
-  explicit NullBuilder(const std::shared_ptr<DataType>& type,
+
+  explicit NullBuilder(const std::shared_ptr<DataType>& ARROW_ARG_UNUSED(type),
                        MemoryPool* pool = default_memory_pool(),
                        int64_t alignment = kDefaultBufferAlignment)
       : NullBuilder(pool, alignment) {}
@@ -77,7 +78,9 @@ class ARROW_EXPORT NullBuilder : public ArrayBuilder {
 
 /// Base class for all Builders that emit an Array of a scalar numerical type.
 template <typename T>
-class NumericBuilder : public ArrayBuilder {
+class NumericBuilder
+    : public ArrayBuilder,
+      public internal::ArrayBuilderExtraOps<NumericBuilder<T>, typename T::c_type> {
  public:
   using TypeClass = T;
   using value_type = typename T::c_type;
@@ -208,6 +211,9 @@ class NumericBuilder : public ArrayBuilder {
   /// \return Status
   Status AppendValues(const std::vector<value_type>& values,
                       const std::vector<bool>& is_valid) {
+    if (values.empty()) {
+      return Status::OK();
+    }
     return AppendValues(values.data(), static_cast<int64_t>(values.size()), is_valid);
   }
 
@@ -215,6 +221,9 @@ class NumericBuilder : public ArrayBuilder {
   /// \param[in] values a std::vector of values
   /// \return Status
   Status AppendValues(const std::vector<value_type>& values) {
+    if (values.empty()) {
+      return Status::OK();
+    }
     return AppendValues(values.data(), static_cast<int64_t>(values.size()));
   }
 
@@ -349,7 +358,9 @@ using DurationBuilder = NumericBuilder<DurationType>;
 
 /// @}
 
-class ARROW_EXPORT BooleanBuilder : public ArrayBuilder {
+class ARROW_EXPORT BooleanBuilder
+    : public ArrayBuilder,
+      public internal::ArrayBuilderExtraOps<BooleanBuilder, bool> {
  public:
   using TypeClass = BooleanType;
   using value_type = bool;
