@@ -22,16 +22,16 @@
 #include "arrow/ipc/writer.h"
 #include "arrow/record_batch.h"
 #include "arrow/result.h"
-#include "arrow/result_internal.h"
 #include "arrow/stl_allocator.h"
 #include "arrow/util/iterator.h"
 #include "arrow/util/logging.h"
-#include "arrow/util/make_unique.h"
 #include "arrow/visit_data_inline.h"
 #include "arrow/visit_type_inline.h"
 
+#include <memory>
+
 #if defined(ARROW_HAVE_NEON) || defined(ARROW_HAVE_SSE4_2)
-#include <xsimd/xsimd.hpp>
+#  include <xsimd/xsimd.hpp>
 #endif
 
 namespace arrow {
@@ -99,7 +99,7 @@ RecordBatchIterator RecordBatchSliceIterator(const RecordBatch& batch,
 }
 
 // Counts the number of quotes in s.
-int64_t CountQuotes(arrow::util::string_view s) {
+int64_t CountQuotes(std::string_view s) {
   return static_cast<int64_t>(std::count(s.begin(), s.end(), '"'));
 }
 
@@ -127,10 +127,20 @@ class ColumnPopulator {
     // Populators are intented to be applied to reasonably small data.  In most cases
     // threading overhead would not be justified.
     ctx.set_use_threads(false);
-    ASSIGN_OR_RAISE(
-        std::shared_ptr<Array> casted,
-        compute::Cast(data, /*to_type=*/utf8(), compute::CastOptions(), &ctx));
-    casted_array_ = checked_pointer_cast<StringArray>(casted);
+    if (data.type() && is_large_binary_like(data.type()->id())) {
+      ARROW_ASSIGN_OR_RAISE(array_, compute::Cast(data, /*to_type=*/large_utf8(),
+                                                  compute::CastOptions(), &ctx));
+    } else {
+      auto casted = compute::Cast(data, /*to_type=*/utf8(), compute::CastOptions(), &ctx);
+      if (casted.ok()) {
+        array_ = std::move(casted).ValueOrDie();
+      } else if (casted.status().IsCapacityError()) {
+        ARROW_ASSIGN_OR_RAISE(array_, compute::Cast(data, /*to_type=*/large_utf8(),
+                                                    compute::CastOptions(), &ctx));
+      } else {
+        return casted.status();
+      }
+    }
     return UpdateRowLengths(row_lengths);
   }
 
@@ -145,7 +155,8 @@ class ColumnPopulator {
 
  protected:
   virtual Status UpdateRowLengths(int64_t* row_lengths) = 0;
-  std::shared_ptr<StringArray> casted_array_;
+  // It must be a `StringArray` or `LargeStringArray`.
+  std::shared_ptr<Array> array_;
   const std::string end_chars_;
   std::shared_ptr<Buffer> null_string_;
 
@@ -155,7 +166,7 @@ class ColumnPopulator {
 
 // Copies the contents of s to out properly escaping any necessary characters.
 // Returns the position next to last copied character.
-char* Escape(arrow::util::string_view s, char* out) {
+char* Escape(std::string_view s, char* out) {
   for (const char c : s) {
     *out++ = c;
     if (c == '"') {
@@ -180,16 +191,29 @@ class UnquotedColumnPopulator : public ColumnPopulator {
         reject_values_with_quotes_(reject_values_with_quotes) {}
 
   Status UpdateRowLengths(int64_t* row_lengths) override {
+    if (ARROW_PREDICT_TRUE(array_->type_id() == Type::STRING)) {
+      return UpdateRowLengths<StringArray>(row_lengths);
+    } else if (ARROW_PREDICT_TRUE(array_->type_id() == Type::LARGE_STRING)) {
+      return UpdateRowLengths<LargeStringArray>(row_lengths);
+    } else {
+      return Status::TypeError("The array must be StringArray or LargeStringArray.");
+    }
+  }
+
+  template <typename StringArrayType>
+  Status UpdateRowLengths(int64_t* row_lengths) {
+    auto casted_array = checked_pointer_cast<StringArrayType>(array_);
     if (reject_values_with_quotes_) {
       // When working on values that, after casting, could produce quotes,
       // we need to return an error in accord with RFC4180.
-      RETURN_NOT_OK(CheckStringArrayHasNoStructuralChars(*casted_array_, delimiter_));
+      RETURN_NOT_OK(CheckStringArrayHasNoStructuralChars<StringArrayType>(*casted_array,
+                                                                          delimiter_));
     }
 
     int64_t row_number = 0;
-    VisitArraySpanInline<StringType>(
-        *casted_array_->data(),
-        [&](arrow::util::string_view s) {
+    VisitArraySpanInline<typename StringArrayType::TypeClass>(
+        *casted_array->data(),
+        [&](std::string_view s) {
           row_lengths[row_number] += static_cast<int64_t>(s.length());
           row_number++;
         },
@@ -201,8 +225,19 @@ class UnquotedColumnPopulator : public ColumnPopulator {
   }
 
   Status PopulateRows(char* output, int64_t* offsets) const override {
+    if (ARROW_PREDICT_TRUE(array_->type_id() == Type::STRING)) {
+      return PopulateRows<StringArray>(output, offsets);
+    } else if (ARROW_PREDICT_TRUE(array_->type_id() == Type::LARGE_STRING)) {
+      return PopulateRows<LargeStringArray>(output, offsets);
+    } else {
+      return Status::TypeError("The array must be StringArray or LargeStringArray.");
+    }
+  }
+
+  template <typename StringArrayType>
+  Status PopulateRows(char* output, int64_t* offsets) const {
     // Function applied to valid values cast to string.
-    auto valid_function = [&](arrow::util::string_view s) {
+    auto valid_function = [&](std::string_view s) {
       memcpy(output + *offsets, s.data(), s.length());
       CopyEndChars(output + *offsets + s.length(), end_chars_.c_str(), end_chars_.size());
       *offsets += static_cast<int64_t>(s.length() + end_chars_.size());
@@ -221,13 +256,14 @@ class UnquotedColumnPopulator : public ColumnPopulator {
       return Status::OK();
     };
 
-    return VisitArraySpanInline<StringType>(*casted_array_->data(), valid_function,
-                                            null_function);
+    return VisitArraySpanInline<typename StringArrayType::TypeClass>(
+        *array_->data(), valid_function, null_function);
   }
 
  private:
   // Returns an error status if string array has any structural characters.
-  static Status CheckStringArrayHasNoStructuralChars(const StringArray& array,
+  template <typename ArrayType>
+  static Status CheckStringArrayHasNoStructuralChars(const ArrayType& array,
                                                      const char delimiter) {
     // scan the underlying string array buffer as a single big string
     const uint8_t* const data = array.raw_data() + array.value_offset(0);
@@ -281,16 +317,28 @@ class QuotedColumnPopulator : public ColumnPopulator {
       : ColumnPopulator(pool, std::move(end_chars), std::move(null_string)) {}
 
   Status UpdateRowLengths(int64_t* row_lengths) override {
-    const StringArray& input = *casted_array_;
+    if (ARROW_PREDICT_TRUE(array_->type_id() == Type::STRING)) {
+      return UpdateRowLengths<StringArray>(row_lengths);
+    } else if (ARROW_PREDICT_TRUE(array_->type_id() == Type::LARGE_STRING)) {
+      return UpdateRowLengths<LargeStringArray>(row_lengths);
+    } else {
+      return Status::TypeError("The array must be StringArray or LargeStringArray.");
+    }
+  }
 
-    row_needs_escaping_.resize(casted_array_->length(), false);
+  template <typename StringArrayType>
+  Status UpdateRowLengths(int64_t* row_lengths) {
+    auto casted_array = checked_pointer_cast<StringArrayType>(array_);
+    const StringArrayType& input = *casted_array;
+
+    row_needs_escaping_.resize(casted_array->length(), false);
 
     if (NoQuoteInArray(input)) {
       // fast path if no quote
       int row_number = 0;
-      VisitArraySpanInline<StringType>(
+      VisitArraySpanInline<typename StringArrayType::TypeClass>(
           *input.data(),
-          [&](arrow::util::string_view s) {
+          [&](std::string_view s) {
             row_lengths[row_number] += static_cast<int64_t>(s.length()) + kQuoteCount;
             row_number++;
           },
@@ -300,9 +348,9 @@ class QuotedColumnPopulator : public ColumnPopulator {
           });
     } else {
       int row_number = 0;
-      VisitArraySpanInline<StringType>(
+      VisitArraySpanInline<typename StringArrayType::TypeClass>(
           *input.data(),
-          [&](arrow::util::string_view s) {
+          [&](std::string_view s) {
             // Each quote in the value string needs to be escaped.
             int64_t escaped_count = CountQuotes(s);
             row_needs_escaping_[row_number] = escaped_count > 0;
@@ -319,10 +367,21 @@ class QuotedColumnPopulator : public ColumnPopulator {
   }
 
   Status PopulateRows(char* output, int64_t* offsets) const override {
+    if (ARROW_PREDICT_TRUE(array_->type_id() == Type::STRING)) {
+      return PopulateRows<StringArray>(output, offsets);
+    } else if (ARROW_PREDICT_TRUE(array_->type_id() == Type::LARGE_STRING)) {
+      return PopulateRows<LargeStringArray>(output, offsets);
+    } else {
+      return Status::TypeError("The array must be StringArray or LargeStringArray.");
+    }
+  }
+
+  template <typename StringArrayType>
+  Status PopulateRows(char* output, int64_t* offsets) const {
     auto needs_escaping = row_needs_escaping_.begin();
-    VisitArraySpanInline<StringType>(
-        *(casted_array_->data()),
-        [&](arrow::util::string_view s) {
+    VisitArraySpanInline<typename StringArrayType::TypeClass>(
+        *array_->data(),
+        [&](std::string_view s) {
           // still needs string content length to be added
           char* row = output + *offsets;
           *row++ = '"';
@@ -354,7 +413,8 @@ class QuotedColumnPopulator : public ColumnPopulator {
 
  private:
   // Returns true if there's no quote in the string array
-  static bool NoQuoteInArray(const StringArray& array) {
+  template <typename StringArrayType>
+  static bool NoQuoteInArray(const StringArrayType& array) {
     const uint8_t* data = array.raw_data() + array.value_offset(0);
     const int64_t buffer_size = array.total_values_length();
     return std::memchr(data, '"', buffer_size) == nullptr;
@@ -366,81 +426,67 @@ class QuotedColumnPopulator : public ColumnPopulator {
   std::vector<bool> row_needs_escaping_;
 };
 
-struct PopulatorFactory {
-  template <typename TypeClass>
-  enable_if_t<is_base_binary_type<TypeClass>::value ||
-                  std::is_same<FixedSizeBinaryType, TypeClass>::value,
-              Status>
-  Visit(const TypeClass& type) {
-    // Determine what ColumnPopulator to use based on desired CSV quoting style.
-    switch (quoting_style) {
-      case QuotingStyle::None:
-        // In unquoted output we must reject values with quotes. Since these types can
-        // produce quotes in their output rendering, we must check them and reject if
-        // quotes appear, hence reject_values_with_quotes is set to true.
-        populator = new UnquotedColumnPopulator(pool, end_chars, delimiter, null_string,
-                                                /*reject_values_with_quotes=*/true);
-        break;
-        // Quoting is needed for strings/binary, or when all valid values need to be
-        // quoted.
-      case QuotingStyle::Needed:
-      case QuotingStyle::AllValid:
-        populator = new QuotedColumnPopulator(pool, end_chars, null_string);
-        break;
+Result<std::unique_ptr<ColumnPopulator>> MakePopulator(
+    const DataType& type, const std::string& end_chars, const char delimiter,
+    const std::shared_ptr<Buffer>& null_string, QuotingStyle quoting_style,
+    MemoryPool* pool) {
+  auto make_populator =
+      [&](const auto& type) -> Result<std::unique_ptr<ColumnPopulator>> {
+    using Type = std::decay_t<decltype(type)>;
+
+    if constexpr (is_primitive_ctype<Type>::value || is_decimal_type<Type>::value ||
+                  is_null_type<Type>::value || is_temporal_type<Type>::value) {
+      switch (quoting_style) {
+        // These types are assumed not to produce any quotes, so we do not need to
+        // check and reject for potential quotes in the casted values in case the
+        // QuotingStyle is None.
+        case QuotingStyle::None:
+          [[fallthrough]];
+        case QuotingStyle::Needed:
+          return std::make_unique<UnquotedColumnPopulator>(
+              pool, end_chars, delimiter, null_string,
+              /*reject_values_with_quotes=*/false);
+        case QuotingStyle::AllValid:
+          return std::make_unique<QuotedColumnPopulator>(pool, end_chars, null_string);
+      }
     }
-    return Status::OK();
-  }
 
-  template <typename TypeClass>
-  enable_if_dictionary<TypeClass, Status> Visit(const TypeClass& type) {
-    return VisitTypeInline(*type.value_type(), this);
-  }
+    if constexpr (is_base_binary_type<Type>::value ||
+                  std::is_same<Type, FixedSizeBinaryType>::value) {
+      // Determine what ColumnPopulator to use based on desired CSV quoting style.
+      switch (quoting_style) {
+        case QuotingStyle::None:
+          // In unquoted output we must reject values with quotes. Since these types
+          // can produce quotes in their output rendering, we must check them and
+          // reject if quotes appear, hence reject_values_with_quotes is set to true.
+          return std::make_unique<UnquotedColumnPopulator>(
+              pool, end_chars, delimiter, null_string,
+              /*reject_values_with_quotes=*/true);
+          // Quoting is needed for strings/binary, or when all valid values need to be
+          // quoted.
+        case QuotingStyle::Needed:
+          [[fallthrough]];
+        case QuotingStyle::AllValid:
+          return std::make_unique<QuotedColumnPopulator>(pool, end_chars, null_string);
+      }
+    }
+
+    if constexpr (std::is_same<Type, DictionaryType>::value) {
+      return MakePopulator(*type.value_type(), end_chars, delimiter, null_string,
+                           quoting_style, pool);
+    }
 
-  template <typename TypeClass>
-  enable_if_t<is_nested_type<TypeClass>::value || is_extension_type<TypeClass>::value,
-              Status>
-  Visit(const TypeClass& type) {
     return Status::Invalid("Unsupported Type:", type.ToString());
-  }
-
-  template <typename TypeClass>
-  enable_if_t<is_primitive_ctype<TypeClass>::value || is_decimal_type<TypeClass>::value ||
-                  is_null_type<TypeClass>::value || is_temporal_type<TypeClass>::value,
-              Status>
-  Visit(const TypeClass& type) {
-    // Determine what ColumnPopulator to use based on desired CSV quoting style.
-    switch (quoting_style) {
-        // These types are assumed not to produce any quotes, so we do not need to check
-        // and reject for potential quotes in the casted values in case the QuotingStyle
-        // is None.
-      case QuotingStyle::None:
-      case QuotingStyle::Needed:
-        populator = new UnquotedColumnPopulator(pool, end_chars, delimiter, null_string,
-                                                /*reject_values_with_quotes=*/false);
-        break;
-      case QuotingStyle::AllValid:
-        populator = new QuotedColumnPopulator(pool, end_chars, null_string);
-        break;
-    }
-    return Status::OK();
-  }
-
-  const std::string end_chars;
-  const char delimiter;
-  std::shared_ptr<Buffer> null_string;
-  const QuotingStyle quoting_style;
-  MemoryPool* pool;
-  ColumnPopulator* populator;
-};
+  };
+  return VisitType(type, make_populator);
+}
 
 Result<std::unique_ptr<ColumnPopulator>> MakePopulator(
-    const Field& field, std::string end_chars, char delimiter,
-    std::shared_ptr<Buffer> null_string, QuotingStyle quoting_style, MemoryPool* pool) {
-  PopulatorFactory factory{std::move(end_chars), delimiter, std::move(null_string),
-                           quoting_style,        pool,      nullptr};
-
-  RETURN_NOT_OK(VisitTypeInline(*field.type(), &factory));
-  return std::unique_ptr<ColumnPopulator>(factory.populator);
+    const Field& field, const std::string& end_chars, char delimiter,
+    const std::shared_ptr<Buffer>& null_string, QuotingStyle quoting_style,
+    MemoryPool* pool) {
+  return MakePopulator(*field.type(), end_chars, delimiter, null_string, quoting_style,
+                       pool);
 }
 
 class CSVWriterImpl : public ipc::RecordBatchWriter {
@@ -454,8 +500,8 @@ class CSVWriterImpl : public ipc::RecordBatchWriter {
       return Status::Invalid("Null string cannot contain quotes.");
     }
 
-    ASSIGN_OR_RAISE(std::shared_ptr<Buffer> null_string,
-                    arrow::AllocateBuffer(options.null_string.length()));
+    ARROW_ASSIGN_OR_RAISE(std::shared_ptr<Buffer> null_string,
+                          arrow::AllocateBuffer(options.null_string.length()));
     memcpy(null_string->mutable_data(), options.null_string.data(),
            options.null_string.length());
 
@@ -464,7 +510,7 @@ class CSVWriterImpl : public ipc::RecordBatchWriter {
     for (int col = 0; col < schema->num_fields(); col++) {
       const std::string& end_chars =
           col < schema->num_fields() - 1 ? delimiter : options.eol;
-      ASSIGN_OR_RAISE(
+      ARROW_ASSIGN_OR_RAISE(
           populators[col],
           MakePopulator(*schema->field(col), end_chars, options.delimiter, null_string,
                         options.quoting_style, options.io_context.pool()));
@@ -481,7 +527,7 @@ class CSVWriterImpl : public ipc::RecordBatchWriter {
   Status WriteRecordBatch(const RecordBatch& batch) override {
     RecordBatchIterator iterator = RecordBatchSliceIterator(batch, options_.batch_size);
     for (auto maybe_slice : iterator) {
-      ASSIGN_OR_RAISE(std::shared_ptr<RecordBatch> slice, maybe_slice);
+      ARROW_ASSIGN_OR_RAISE(std::shared_ptr<RecordBatch> slice, maybe_slice);
       RETURN_NOT_OK(TranslateMinimalBatch(*slice));
       RETURN_NOT_OK(sink_->Write(data_buffer_));
       stats_.num_record_batches++;
@@ -523,10 +569,11 @@ class CSVWriterImpl : public ipc::RecordBatchWriter {
   Status PrepareForContentsWrite() {
     // Only called once, as part of initialization
     if (data_buffer_ == nullptr) {
-      ASSIGN_OR_RAISE(data_buffer_,
-                      AllocateResizableBuffer(
-                          options_.batch_size * schema_->num_fields() * kColumnSizeGuess,
-                          options_.io_context.pool()));
+      ARROW_ASSIGN_OR_RAISE(
+          data_buffer_,
+          AllocateResizableBuffer(
+              options_.batch_size * schema_->num_fields() * kColumnSizeGuess,
+              options_.io_context.pool()));
     }
     return Status::OK();
   }
@@ -618,24 +665,24 @@ class CSVWriterImpl : public ipc::RecordBatchWriter {
 
 Status WriteCSV(const Table& table, const WriteOptions& options,
                 arrow::io::OutputStream* output) {
-  ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(output, table.schema(), options));
+  ARROW_ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(output, table.schema(), options));
   RETURN_NOT_OK(writer->WriteTable(table));
   return writer->Close();
 }
 
 Status WriteCSV(const RecordBatch& batch, const WriteOptions& options,
                 arrow::io::OutputStream* output) {
-  ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(output, batch.schema(), options));
+  ARROW_ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(output, batch.schema(), options));
   RETURN_NOT_OK(writer->WriteRecordBatch(batch));
   return writer->Close();
 }
 
 Status WriteCSV(const std::shared_ptr<RecordBatchReader>& reader,
                 const WriteOptions& options, arrow::io::OutputStream* output) {
-  ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(output, reader->schema(), options));
+  ARROW_ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(output, reader->schema(), options));
   std::shared_ptr<RecordBatch> batch;
   while (true) {
-    ASSIGN_OR_RAISE(batch, reader->Next());
+    ARROW_ASSIGN_OR_RAISE(batch, reader->Next());
     if (batch == nullptr) break;
     RETURN_NOT_OK(writer->WriteRecordBatch(*batch));
   }
